package pwc.taxtech.atms.vat.service.impl.report.functions;

import com.alibaba.fastjson.JSON;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.EvaluationException;
import org.apache.poi.ss.formula.eval.OperandResolver;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.vat.entity.DataSource;
import pwc.taxtech.atms.vat.entity.DataSourceDetail;
import pwc.taxtech.atms.vat.entity.PeriodFormulaBlock;
import pwc.taxtech.atms.vat.service.impl.FormulaAgent;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

public class FunctionBase {
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    protected FormulaContext formulaContext;
    final FormulaAgent agent;

    public FunctionBase(FormulaContext formulaContext) {
        this.formulaContext = formulaContext;
        this.agent = formulaContext.getFormulaAgent();
    }

    public String getStringParam(ValueEval valueEval, OperationEvaluationContext ec) {
        try {
            ValueEval v1 = OperandResolver.getSingleValue(valueEval,
                    ec.getRowIndex(),
                    ec.getColumnIndex());
            return OperandResolver.coerceValueToString(v1);
        } catch (EvaluationException e) {
            e.printStackTrace();
            return null;
        }
    }

    public Integer getIntParam(ValueEval valueEval, OperationEvaluationContext ec) {
        try {
            ValueEval v1 = OperandResolver.getSingleValue(valueEval,
                    ec.getRowIndex(),
                    ec.getColumnIndex());
            return OperandResolver.coerceValueToInt(v1);
        } catch (EvaluationException e) {
            e.printStackTrace();
            return null;
        }
    }

    public Long getCellTemplateID(int period, OperationEvaluationContext ec) {
        return FormulaHelper
                .getCellTemplateIDWithTemplateCodeAndColumnIndexAndRowIndex(ec.getWorkbook().getSheetName(ec.getSheetIndex()), ec.getColumnIndex(), ec.getRowIndex(), period, formulaContext);
    }

    public int getYear(int parameterYear) {
        if (parameterYear <= -1) {
            return parameterYear + formulaContext.getYear();
        } else if (parameterYear == 0) {
            return formulaContext.getYear();
        } else {
            return parameterYear;
        }
    }

    public int getPeriod(int parameterPeriod, int[] yearOffset) {
        if (yearOffset.length == 0) {
            yearOffset = new int[10];
        }
        yearOffset[0] = 0;
        if (parameterPeriod == -99) {
            return parameterPeriod;
        } else if (parameterPeriod <= -1) {
            int period = parameterPeriod + formulaContext.getPeriod();
            while (period <= 0) {
                yearOffset[0]--;
                period += 12;
            }
            return period;
        } else if (parameterPeriod == 0) {
            return formulaContext.getPeriod();
        } else {
            return parameterPeriod;
        }
    }

    public Long saveDataSource(OperationEvaluationContext ec, List<Object> dataSourceList,
                               FormulaDataSourceDetailType formulaDataSourceDetailType, BigDecimal val) {
        Date creatime = new Date();
        DataSource dataSource = new DataSource();
        Long dataSourceID = SpringContextUtil.distributedIDService.nextId();
        dataSource.setId(dataSourceID);
        dataSource.setType(1);
        dataSource.setName("ReportDataSource");
        dataSource.setAmount(val);
        dataSource.setDescription("");
        dataSource.setCreateBy("Admin");
        dataSource.setUpdateBy("Admin");
        dataSource.setCreateTime(creatime);
        dataSource.setUpdateTime(creatime);
        dataSource.setRowIndex(ec.getRowIndex());
        dataSource.setColumnIndex(ec.getColumnIndex());
        dataSource.setRowName("");
        dataSource.setColumnName("");
        SpringContextUtil.dataSourceMapper.insertSelective(dataSource);
        for (Object obj : dataSourceList) {
            DataSourceDetail dataSourceDetail = new DataSourceDetail();
            dataSourceDetail.setId(SpringContextUtil.distributedIDService.nextId());
            dataSourceDetail.setDataSourceId(dataSourceID);
            dataSourceDetail.setDataSourceType(formulaDataSourceDetailType.getCode());
            dataSourceDetail.setItemValue(JSON.toJSONString(obj));
            SpringContextUtil.dataSourceDetailMapper.insertSelective(dataSourceDetail);
        }

        return dataSourceID;
    }

    public void saveFormulaBlock(int period, OperationEvaluationContext ec,
                                 String formulaExpression, BigDecimal val, Long dataSourceID) {
        Long cellTemplateID = getCellTemplateID(period, ec);
        Date creatime = new Date();
        PeriodFormulaBlock periodFormulaBlock = new PeriodFormulaBlock();
        periodFormulaBlock.setId(SpringContextUtil.distributedIDService.nextId());
        periodFormulaBlock.setPeriod(period);
        periodFormulaBlock.setReportId(0L);
        periodFormulaBlock.setCellTemplateId(cellTemplateID);
        periodFormulaBlock.setFormulaExpression(formulaExpression);
        periodFormulaBlock.setData(val.toString());
        periodFormulaBlock.setDataSourceId(dataSourceID);
        periodFormulaBlock.setCreateBy("Admin");
        periodFormulaBlock.setCreateTime(creatime);
        periodFormulaBlock.setUpdateBy("Admin");
        periodFormulaBlock.setUpdateTime(creatime);
        SpringContextUtil.periodFormulaBlockMapper.insertSelective(periodFormulaBlock);
    }

}
