package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.constant.Constant;

import java.math.BigDecimal;


/// <summary>
/// 名称：进项发票明细
/// 功能：从发票取数，一般取发票金额或税额汇总数
/// </summary>
/// <param name="taxRate">税率——取值为17%，13%，11%，6%, 5%，3%，1.5%, 其他(其他表示除了17%,13%,11%,6%,5%,3%的税率)</param>
/// <param name="goodsName">货物名称——精确匹配引号中的值，99代表所有值（包括空值）</param>
/// <param name="amountType">取值——两种取值结果“金额” 0 和“税额” 1</param>
/// <param name="period">会计期间——三种取值结果 “-1”指上期，“0”当期，“99”当前累计（从期初到目前）</param>
/// <returns></returns>
public class JXFPMX extends FunctionBase implements FreeRefFunction {
    public JXFPMX(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length < 4) {
            return null;
        }

        String taxRate = getStringParam(args[0], ec);
        String goodsName = getStringParam(args[1], ec);
        int amountType = getIntParam(args[2], ec);
        int period = getIntParam(args[3], ec);

        String formulaExpression = "JXFPMX(\"" + taxRate + "\",\"" + goodsName + "\","
                + amountType + "," + period + ")";
        logger.debug(formulaExpression);

        int curYear = formulaContext.getYear();
        int[] yearOffSet = {};
        period = getPeriod(period, yearOffSet);
        curYear = curYear + yearOffSet[0];

        BigDecimal taxRateVal;
        if (taxRate.equals(Constant.Other)) {
            taxRateVal = new BigDecimal("-1");
        } else if (!taxRate.endsWith("%")) {
            if (taxRate.equals("99")) {
                taxRateVal = new BigDecimal("99");
            } else {
                return NumberEval.ZERO;
            }
        } else {
            taxRate = StringUtils.removeEnd(taxRate, "%");
            try {
                taxRateVal = new BigDecimal(Double.parseDouble(taxRate));
            } catch (NumberFormatException ex) {
                return NumberEval.ZERO;
            }

            taxRateVal = taxRateVal.divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);
        }




        return NumberEval.ZERO;
    }
}
